#! /usr/bin/env python3
#

import os
import subprocess
import sys

MOUNT_POINT="/mnt/root"
BOOT_LOG_PATH=MOUNT_POINT + "/var/log/boot.log"
ETC_FSTAB_PATH=MOUNT_POINT + "/etc/fstab"

blk_list=[]


def mount(src, dst):
    try:
        subprocess.check_output("mount {} {}".format(src, dst), shell=True).strip()
    except Exception as e:
        return False
    return True



def umount(dst):
    try:
        subprocess.check_output("umount {}".format(dst), shell=True).strip()
    except Exception as e:
        return False
    return True



def get_fstab():
    try:
        fstab_datas = subprocess.check_output("{}/get_fstab".format(os.path.dirname(os.path.realpath(__file__))), shell=True).strip()
        return bytes.decode(fstab_datas).split("\n")
    except Exception as e:
        return []



def get_blk_info():
    try:
        blks = subprocess.check_output("blkid -o export", shell=True).strip()
        return bytes.decode(blks).split("\n\n")
    except Exception as e:
        return []



def find_last_root():
    if not os.path.exists(MOUNT_POINT):
        os.mkdir(MOUNT_POINT)

    blks = get_blk_info()
    global blk_list
    for blk in blks:
        blk_dir={}
        for data in blk.split("\n"):
            data_tmp = data.split("=")
            if data_tmp[0] == "DEVNAME":
                blk_dir.update({'devname':data_tmp[1]})
            elif data_tmp[0] == "UUID":
                blk_dir.update({'uuid':data_tmp[1]})
            elif data_tmp[0] == "LABEL":
                blk_dir.update({'label':data_tmp[1]})
        if not blk_dir.get('uuid') or not blk_dir.get('devname'):
            continue
        blk_list.append(blk_dir)

    last_root_time=0
    last_root_uuid=""
    for blk in blk_list:
        if not mount(blk.get('devname'), MOUNT_POINT):
            continue
        try:
            if os.path.exists("{}/etc".format(MOUNT_POINT)):
                root_time = get_var_logtime()
                if root_time and last_root_time < round(root_time):
                    last_root_time = round(root_time)
                    last_root_uuid = blk.get('uuid')
        except Exception as e:
            pass
        finally:
            umount(MOUNT_POINT)
    return last_root_uuid



def get_var_logtime():
    roottime = None
    if os.path.exists(BOOT_LOG_PATH):
        roottime = os.path.getmtime(BOOT_LOG_PATH)
    fstab_datas = get_fstab()
    var_list=[]
    fstab_dict={}
    for fstab_data in fstab_datas:
        if not fstab_data or not fstab_data.split(","):
            continue
        fstab_dict.update({fstab_data.split(",")[0]:fstab_data.split(",")[1]})
        if 'var' in fstab_data:
            var_list.append(fstab_data.split(",")[0])
    for var in var_list:
        var_split = var.split("/")
        count = len(var_split)
        while(1 < count):
            var_partition = '/'.join(var_split[:count])
            roottime = find_var_logtime(var_partition,fstab_dict)
            if roottime:
                return roottime
            count = count - 1
    return roottime



def is_valid_partition(key):
    global blk_list
    for blk in blk_list:
        if key == ("UUID=" + blk.get('uuid', '')) or key == blk.get('devname', '') or key == ("LABEL=" + blk.get('label', '')):
            return True
    return False



def find_var_logtime(var_partition, fstab_dict):
    logtime = None
    for key,value in fstab_dict.items():
        if var_partition == value and is_valid_partition(key):
            if not mount(key, MOUNT_POINT):
                continue
            if os.path.exists(BOOT_LOG_PATH):
               logtime = os.path.getmtime(BOOT_LOG_PATH)
               umount(MOUNT_POINT)
               break
            umount(MOUNT_POINT)
    return logtime




if __name__ == '__main__':
    last_root_uuid = find_last_root()
    if last_root_uuid:
        print(last_root_uuid)
        sys.exit(0)
    else:
        sys.exit(1)
